#!/usr/bin/perl -Tw
###############################################################################
# Click Manager 2.2.5
# Copyright (C) 2003  Aardvark Industries
# http://www.aardvarkind.com/index.php?page=clickmanager
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################

# Load modules and whatnot.
use strict;
use CGI::Carp qw(fatalsToBrowser);
use CGI qw/:standard/;
use Fcntl qw(:DEFAULT :flock);

# Declare variables that will be used later in the script.
my %tmpl;

# Settings that you might need to change.
$tmpl{yoursite} = 'Aardvark Industries';          # The name of your site.
$tmpl{yoururl} = 'http://www.aardvarkind.com/';   # The URL to your site.
my $dir = '.';                                    # The full path to the directory this file is in.
my $require_pw = '0';                             # Require a password to view the stats?  1=yes 0=no
my $pw = 'password';                              # If you require a password, what will the password be?
$tmpl{width} = '600';                             # Width of the bar graph.

# Tells the script what version it is.
$tmpl{version} = '2.2.5';

# Read the form using CGI.pm.
my $q = new CGI;
my %form = map { $_ => $q->param($_) } $q->param;

# Check the password.
if ($require_pw) {
  if ($form{pw} eq $pw) { $tmpl{pw} = $form{pw}; &stats; }
  elsif ($form{pw}) { &error("Invalid password!  Please go back and try again."); }
  else { &login; }
}
else { &stats; }



# Login if a password is required.
sub login {
  print "Content-type: text/html\n\n";
  print <<EndHTML;
<form action="stats.cgi" method="post">
<input name="pw" type="password" size="20" />
<input type="submit" value="Login" />
</form>
Powered By <a href="http://www.aardvarkind.com/?a=cgi&amp;b=clickmanager&amp;c=index" target="_blank">Click Manager $tmpl{version}</a>
EndHTML
}



# The stats and graphs page.
sub stats {
  sysopen(CLICKMANAGER, "$dir/clickmanager.txt", O_RDONLY) || die "$dir/clickmanager.txt\n\n$!, stopped";
  flock(CLICKMANAGER, 2);
  my @db = <CLICKMANAGER>;
  close(CLICKMANAGER);
  sysopen(TOTAL, "$dir/total.txt", O_RDONLY) || die "$dir/total.txt\n\n$!, stopped";
  flock(TOTAL, 2);
  ($tmpl{hits_today}, $tmpl{hits_1}, $tmpl{hits_2}, $tmpl{hits_3}, $tmpl{hits_highest}, $tmpl{hits_total}, $tmpl{hits_days}) = split(/\|/, <TOTAL>);
  close(TOTAL);
  if ($tmpl{hits_days}) { $tmpl{average} = int($tmpl{hits_total} / $tmpl{hits_days} + .5); }
  else { $tmpl{average} = 0; }
  my $max = 1;
  foreach ($tmpl{average}, $tmpl{hits_today}, $tmpl{hits_1}, $tmpl{hits_2}, $tmpl{hits_3}, $tmpl{hits_highest}) {
    if ($_ > $max) { $max = $_; } 
  }
  $tmpl{bar_average} = int($tmpl{average} / $max * $tmpl{width} + .5);
  $tmpl{bar_hits_today} = int($tmpl{hits_today} / $max * $tmpl{width} + .5);
  $tmpl{bar_hits_1} = int($tmpl{hits_1} / $max * $tmpl{width} + .5);
  $tmpl{bar_hits_2} = int($tmpl{hits_2} / $max * $tmpl{width} + .5);
  $tmpl{bar_hits_3} = int($tmpl{hits_3} / $max * $tmpl{width} + .5);
  $tmpl{bar_hits_highest} = int($tmpl{hits_highest} / $max * $tmpl{width} + .5);
  if ($form{sort} eq "id") {
    foreach (@db) {
      my($id, $url, $hits) = split(/\|/);
      $tmpl{individual} .= qq~<tr><td class="cell">$id</td><td class="cell"><a href="$url" target="_blank">$url</a></td><td align="right" class="cell">$hits</td></tr>~;
    }
  }
  elsif ($form{sort} eq "url") {
    foreach my $line(@db) {
      chomp;
      my($id, $url, $hits) = split(/\|/, $line);
      $line = join('|', $url, $hits, $id);
    }
    @db = sort { $a cmp $b }@db;
    foreach (@db) {
      my($url, $hits, $id) = split(/\|/);
      $tmpl{individual} .= qq~<tr><td class="cell">$id</td><td class="cell"><a href="$url" target="_blank">$url</a></td><td align="right" class="cell">$hits</td></tr>~;
    }
  }
  else {
    foreach my $line(@db) {
      chomp;
      my($id, $url, $hits) = split(/\|/, $line);
      $line = join('|', $hits, $url, $id);
    }
    @db = sort { $b <=> $a }@db;
    foreach (@db) {
      my($hits, $url, $id) = split(/\|/);
      if (length($url) >= 75) {
        $id = length($url);
        $url =~ /(.*)(....................)/;
        $url = &truncate_string($1, 53) . $2;
      }
      $tmpl{individual} .= qq~<tr><td class="cell">$id</td><td class="cell"><a href="clickmanager.cgi?dl=$id" target="_blank">$url</a></td><td align="right" class="cell">$hits</td></tr>~;
    }
  }
  &template;
}



# Used for some error messages.
sub error {
  print "Content-type: text/html\n\n$_[0]\n";
  exit;
}



# Reads and parses the template.
sub template {
  open(TEMPLATE, "$dir/template.html") || die "$dir/template.html\n\n$!, stopped";
  flock(TEMPLATE, 2);
  my $template = join('', <TEMPLATE>);
  close(TEMPLATE);
  # If you feel the need to change or delete the following link, you're truly an ass.
  # Changing or deleting it also violates the license, so it's illegal to change it.
  $tmpl{copyright} = "<a href=\"http://www.aardvarkind.com/index.php?page=clickmanager\" target=\"_blank\">Click Manager $tmpl{version}</a> is  2001-2002 <a href=\"http://www.aardvarkind.com/\" target=\"_blank\">Aardvark Industries</a>.  All rights reserved.";
  print "Content-type: text/html\n\n";
  if ($template =~ /<\$copyright>/) {
    $template =~ s/<\$(.+?)>/$tmpl{$1}/ig;
    print $template;
  }
  else { print "Please put the copyright link back in the template.\n"; }
}



# Truncates a string.
sub truncate_string { 
    return (length($_[0]) > $_[1]) ? substr($_[0],0,$_[1]+1).'...' : $_[0]; 
    
}